//
//  ClockViewController.m
//  Clock
//
//  Created by aka on 08/12/20.
//  Copyright Masayuki Akamatsu 2008. All rights reserved.
//

#import "ClockViewController.h"

@implementation ClockViewController

// クラスの初期化
+ (void)initialize
{
	// ユーザ・デフォルトの初期設定
	// 初期値の辞書を作成
	NSMutableDictionary *defaultValues = [NSMutableDictionary dictionary]; 
	// 初期値を設定
	[defaultValues setValue:[NSNumber numberWithBool:NO] forKey:kAlarmEnabledKey]; 
	[defaultValues setValue:[NSNumber numberWithFloat:0.0] forKey: kAlarmHourKey]; 
	// ユーザ・デフォルトを取得
	NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
	// ユーザ・デフォルトに初期値を登録
	[userDefaults registerDefaults:defaultValues];  
}

/*
 // The designated initializer. Override to perform setup that is required before the view is loaded.
 - (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
 if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
 // Custom initialization
 }
 return self;
 }
 */

/*
 // Implement loadView to create a view hierarchy programmatically, without using a nib.
 - (void)loadView {
 }
 */


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
// ビューの読み込みが完了した時に呼び出されるメソッド
- (void)viewDidLoad
{
	// スーパー・クラスのメソッドを呼び出す
	[super viewDidLoad];
	
	// タイマーの作成（動作開始）
	[NSTimer scheduledTimerWithTimeInterval:1.0		// 時間間隔（秒）
									 target:self	// 呼び出すオブジェクト
								   selector:@selector(driveClock:)	// 呼び出すメソッド
								   userInfo:nil		// ユーザ利用の情報オブジェクト
									repeats:YES];	// 繰り返し
	
	// アラーム音の再生を準備
	NSString *path = [[NSBundle mainBundle] pathForResource:@"Alarm" ofType:@"caf"];
	NSURL *url = [NSURL fileURLWithPath:path]; 
	alarmPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL:url error:nil];
	alarmPlayer.numberOfLoops = -1; // ループ再生
	
	// 自動ロックの禁止
	UIApplication *application = [UIApplication sharedApplication];
	application.idleTimerDisabled = YES;
	
	// ユーザ・デフォルトを取得
	NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
	// ユーザ・デフォルトから設定値を取得
	alarmEnabled = [userDefaults boolForKey:kAlarmEnabledKey];
	alarmHour = [userDefaults floatForKey:kAlarmHourKey];
	// ユーザ・デフォルトによる設定
	[self setAlarmItems];
}

// タイマーから呼び出されるメソッド
- (void)driveClock:(NSTimer *)timer
{
	// 現在時刻を取得
	NSDate *today = [NSDate date];
	// 現在時刻の時、分、秒を取得
	NSCalendar *calender = [NSCalendar currentCalendar];
	unsigned flags = NSHourCalendarUnit | NSMinuteCalendarUnit | NSSecondCalendarUnit;
	NSDateComponents *todayComponents = [calender components:flags fromDate:today];
	int hour = [todayComponents hour];
	int min = [todayComponents minute];
	int sec = [todayComponents second];
	float fineHour = (hour % 12) + min / 60.0;
	
	// 時針、分針、秒針の回転
	hourHand.transform = CGAffineTransformMakeRotation(M_PI * 2 * fineHour / 12.0);
	minuteHand.transform = CGAffineTransformMakeRotation(M_PI * 2 * min / 60.0);
	secondHand.transform = CGAffineTransformMakeRotation(M_PI * 2 * sec / 60.0);
	
	// アラームの処理
	if (alarmEnabled)
	{
		float difference = fineHour - alarmHour;
		if (difference >= 0.0 && difference < 0.1)
		{
			if (!alarmPlayer.playing)
				[alarmPlayer play];
		}
		else
		{
			if (alarmPlayer.playing)
				[alarmPlayer stop];
		}
	}
}

// アラームのビュー・アイテムを設定
- (void)setAlarmItems
{
	// アラーム・ボタンを設定
	alarmButton.selected = alarmEnabled;
	// アラーム針を設定
	alarmHand.transform = CGAffineTransformMakeRotation(M_PI * 2 * alarmHour / 12.0);
}


// アラームボタンがタップされた時に呼び出されるメソッド
- (IBAction)toggleAlarmButton:(UIButton *)sender
{
	// アラームの状態を反転
	alarmEnabled = !alarmEnabled;

	// アラーム状態がオフであれば、アラームを停止
	if (alarmEnabled == NO)
		if (alarmPlayer.playing)
			[alarmPlayer stop];		
	
	// アラームのビュー・アイテムを設定
	[self setAlarmItems];
}


// 盤面の透明ボタンがタップやドラッグされた時に呼び出されるメソッド
- (IBAction)moveAlarmHand:(UIButton *)sender forEvent:(UIEvent *)event
{
	// タップされた座標を取得
	NSSet *touches = [event touchesForView:sender];
	UITouch *touch = [touches anyObject];
	CGPoint touchPoint = [touch locationInView:sender];
	
	// タップされた座標に対応するアラーム時刻を算定
	float angle = atan2(150 - touchPoint.y, touchPoint.x - 150);
	alarmHour = (M_PI * 0.5 - angle) * 12.0 / (M_PI * 2);
	if (alarmHour < 0.0)
		alarmHour += 12.0;
	
	// アラームのビュー・アイテムを設定
	[self setAlarmItems];
}

// ビューが閉じられようとする時に呼び出されるメソッド
- (void)viewWillDisappear:(BOOL)animated
{
	// ユーザ・デフォルトを取得
	NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
	// ユーザ・デフォルトに現在の値を設定
	[userDefaults setBool:alarmEnabled forKey:kAlarmEnabledKey];
	[userDefaults setFloat:alarmHour forKey:kAlarmHourKey];
	// ユーザ・デフォルトの保存
	[userDefaults synchronize];
}


/*
 // Override to allow orientations other than the default portrait orientation.
 - (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
 // Return YES for supported orientations
 return (interfaceOrientation == UIInterfaceOrientationPortrait);
 }
 */


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning]; // Releases the view if it doesn't have a superview
    // Release anything that's not essential, such as cached data
}


- (void)dealloc
{	
    [super dealloc];
}

@end
